using Microsoft.Extensions.Logging;

namespace FitConnect.Encryption;

// TODO: Throw useful error on macOS to add environment variable DYLD_LIBRARY_PATH to the path.

public record KeySet(string? PrivateKeyDecryption,
    string? PrivateKeySigning, string? PublicKeyEncryption, string? PublicKeySignatureVerification
);

public class FitEncryption {
    private readonly ILogger? _logger;
    public string? PrivateKeyDecryption { get; set; }
    public string? PrivateKeySigning { get; set; }
    public string? PublicKeyEncryption { get; set; }
    public string? PublicKeySignatureVerification { get; set; }

    private readonly IEncryptor _encryptor = new JoseEncryptor();

    public FitEncryption(ILogger? logger) {
        _logger = logger;
    }

    public FitEncryption(string? privateKeyDecryption, string? privateKeySigning,
        string? publicKeyEncryption, string? publicKeySignatureVerification, ILogger? logger) :
        this(logger) {
        PrivateKeyDecryption = privateKeyDecryption;
        PrivateKeySigning = privateKeySigning;
        PublicKeyEncryption = publicKeyEncryption;
        PublicKeySignatureVerification = publicKeySignatureVerification;
    }

    public FitEncryption(
        KeySet keySet,
        ILogger? logger) : this(logger) {
        PrivateKeyDecryption = keySet.PrivateKeyDecryption;
        PrivateKeySigning = keySet.PrivateKeySigning;
        PublicKeyEncryption = keySet.PublicKeyEncryption;
        PublicKeySignatureVerification = keySet.PublicKeySignatureVerification;
    }

    public (string plainText, byte[] plainBytes, byte[] tag) Decrypt(string cypherText, string key) =>
        _encryptor.Decrypt(key, cypherText);

    public (string plainText, byte[] plainBytes, byte[] tag) Decrypt(string cypherText) {
        if (PrivateKeyDecryption == null) {
            throw new InvalidOperationException("PrivateKey is not provided");
        }

        return Decrypt(cypherText, PrivateKeyDecryption);
    }


    public string Encrypt(string plainText, string key) => _encryptor.Encrypt(key, plainText);

    public string Encrypt(string plain) {
        if (PublicKeyEncryption == null) {
            throw new InvalidOperationException("PrivateKey is not provided");
        }

        return Encrypt(plain, PublicKeyEncryption);
    }

    public string Encrypt(byte[] plain, string key) => _encryptor.Encrypt(key, plain);

    public string Encrypt(byte[] plain) {
        if (PublicKeyEncryption == null) {
            throw new InvalidOperationException("PrivateKey is not provided");
        }

        return Encrypt(plain, PublicKeyEncryption);
    }
}
