using System.Security.Authentication;
using System.Security.Cryptography.X509Certificates;
using System.Text.RegularExpressions;
using FitConnect.Interfaces;
using FitConnect.Models;
using FitConnect.Services;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1;
using Microsoft.Extensions.Logging;

namespace FitConnect;

/// <summary>
///     The fluent implementation for the <see cref="Sender" /> to encapsulate the FitConnect API.
/// </summary>
/// <example>
///     Reference for FluentSender
///     <code>
/// client.Sender
///     .Authenticate(clientId!, clientSecret!)
///     .CreateSubmission(new Submission { Attachments = new List() })
///     .UploadAttachments()
///     .SendSubmission(new Metadata(), new Data());
/// </code>
/// </example>
public class FluentSender : Sender, IFluentSender, IFluentSenderWithDestination,
    IFluentSenderWithAttachments, IFluentSenderWithData {
    private State _nextStep = State.Authenticate;
    private OAuthAccessToken? token;

    public FluentSender(FitConnectEndpoints endpoints,
        IOAuthService oAuthService,
        ISubmissionService submissionService,
        IRouteService routeService,
        IDestinationService destinationService,
        ILogger? logger = null) : base(endpoints, oAuthService, submissionService,
        routeService, destinationService, logger) {
    }

    public string PublicKey { get; set; }
    public CreateSubmissionDto? NewSubmission { get; set; }
    public Submission Submission { get; set; }


    public IFluentSenderWithDestination FindDestinationId(string leiaKey, string? ags, string? ars,
        string? areaId) {
        var destinationId = RouteService.GetDestinationIdAsync(leiaKey, ags, ars, areaId).Result;
        return WithDestination(destinationId);
    }

    public IFluentSenderWithDestination FindDestinationId(Destination destination) {
        throw new NotImplementedException();
    }

    public IFluentSenderWithDestination WithDestination(string destinationId) {
        if (!Regex.IsMatch(destinationId, ".*"))
            throw new AggregateException("Invalid destination id");

        Submission = CreateSubmission(destinationId).Result;
        return this;
    }

    public IFluentSenderWithData WithData(string data) {
        Submission.Data = Data.FromString(data);
        return this;
    }

    public Submission Submit() {
        throw new NotImplementedException();
    }


    /// <summary>
    ///     Authenticates with the FitConnect API.
    /// </summary>
    /// <param name="clientId">The client id from the submission service</param>
    /// <param name="clientSecret">The client secret</param>
    /// <param name="scope">Optional scope for the credentials</param>
    /// <returns></returns>
    private FluentSender
        Authenticate(string clientId, string clientSecret, string? scope = null) {
        if (_nextStep != State.Authenticate)
            throw new InvalidOperationException("Client not ready for authentication.");

        token ??= AuthenticateAsync(clientId, clientSecret, scope).Result;
        if (token == null)
            throw new AuthenticationException("Failed to authenticate with FitConnect.");
        
        _nextStep = State.CreateSubmission;
        return this;
    }

    /// <summary>
    ///     Creates a new <see cref="Submission" /> on the FitConnect server.
    /// </summary>
    /// <param name="destinationId">The id of the destination the submission has to be sent to</param>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException">If sender is not authenticated</exception>
    /// <exception cref="ArgumentException">If submission is not ready to be sent</exception>
    private async Task<Submission> CreateSubmission(string destinationId) {
        try {
            Authenticate(Owner.ClientId, Owner.ClientSecret);
        }
        catch (Exception e) {
            throw new AuthenticationException("Failed to authenticate with FitConnect", e);
        }

        if (_nextStep != State.CreateSubmission)
            throw new AuthenticationException(
                "Can only create a submission after authentication");

        var submission = new Submission() {
            DestinationId = destinationId
        };

        DestinationService.Token = token!.AccessToken;
        var publicKey = await DestinationService.GetPublicKey(destinationId);
        PublicKey = publicKey;

        _nextStep = State.UploadAttachments;

        return submission;
    }


    /// <summary>
    ///     Posts the <see cref="Attachment" />s to the FitConnect API. The attachment ids are sent with
    ///     creating the submission.
    /// </summary>
    /// <returns></returns>
    /// <exception cref="InvalidOperationException"></exception>
    public FluentSender UploadAttachments() {
        if (_nextStep != State.UploadAttachments)
            throw new InvalidOperationException(
                "Can only upload attachments after a submission has been created");
        if (NewSubmission == null)
            throw new InvalidOperationException("You must create a submission first.");

        _nextStep = State.SendSubmission;
        if (Submission.Attachments.Count == 0)
            return this;

        var encryptedAttachments = Encrypt(PublicKey, Submission.Attachments);
        UploadAttachmentsAsync(encryptedAttachments).Wait();
        return this;
    }

    /// <summary>
    ///     Sending the submission and finishing the process.
    ///     <para>
    ///         Finishing the submission by transmitting <see cref="Metadata" /> and <see cref="Data" />.
    ///     </para>
    /// </summary>
    /// <param name="metadata"></param>
    /// <param name="data"></param>
    /// <returns></returns>
    /// <remarks>This call finishes the submission process</remarks>
    /// <exception cref="InvalidOperationException"></exception>
    // public FluentSender SendSubmission(Metadata? metadata, Data? data = null) {
    //     if (_nextStep != State.SendSubmission)
    //         throw new InvalidOperationException(
    //             "Can only send a submission after attachments have been uploaded");
    //
    //     if (NewSubmission == null)
    //         throw new InvalidOperationException("You must create a submission first.");
    //
    //     Submission.Metadata = metadata;
    //     Submission.Data = data;
    //     var submitSubmissionDto = CreateSubmitSubmissionDto(Submission);
    //     SubmissionService.SubmitSubmission(Submission.Id, submitSubmissionDto);
    //
    //     _nextStep = State.Complete;
    //
    //     return this;
    // }
    public IFluentSenderWithAttachments WithAttachments(IEnumerable<Attachment> attachments) {
        Submission.Attachments = attachments.ToList();
        return this;
    }


    private enum State {
        Authenticate,
        CreateSubmission,
        UploadAttachments,
        SendSubmission,
        Complete
    }

    IFluentSender IFluentSenderReady.Submit() {
        Logger?.LogInformation("Sending submission");
        return this;
    }
}
